@@ -1,5 +1,5 @@
 #
-# Copyright 2008-2013 Universidad Complutense de Madrid
+# Copyright 2008-2014 Universidad Complutense de Madrid
 # 
 # This file is part of Numina
 # 
@@ -204,6 +204,11 @@ def correct_flatfield(data, flat, dtype='float32'):
     result = result.astype(dtype)
     return result
 
+def correct_sky(data, sky, dtype='float32'):
+    result = data - sky
+    result = result.astype(dtype)
+    return result
+
 def correct_nonlinearity(data, polynomial, dtype='float32'):
     result = numpy.polyval(polynomial, data)
     result = result.astype(dtype)
