@@ -1,5 +1,5 @@
 #
-# Copyright 2008-2013 Universidad Complutense de Madrid
+# Copyright 2008-2014 Universidad Complutense de Madrid
 # 
 # This file is part of Numina
 # 
@@ -21,44 +21,27 @@
 Basic Data Products
 '''
 
-import warnings
+
+from astropy.io import fits
 
 class DataFrame(object):
-    def __init__(self, frame=None, filename=None, itype='UNKNOWN'):
+    '''A handle to a image in disk or in memory.'''
+    def __init__(self, frame=None, filename=None):
         if frame is None and filename is None:
             raise ValueError('only one in frame and filename can be None') 
         self.frame = frame
         self.filename = filename
-        self.itype = itype
 
-    def __getstate__(self):
-        if self.frame is None and self.filename is None:
-            raise ValueError('only one in frame and filename can be None') 
-        # save fits file
-        if self.frame is None: 
-            # assume filename contains a FITS file
-            return {'filename': self.filename}
+    def open(self):
+        if self.frame is None:
+            return fits.open(self.filename, memmap=True, mode='readonly')
         else:
-            if self.filename:
-                filename = self.filename
-            elif self.frame[0].header.has_key('FILENAME'):
-                filename = self.frame[0].header['FILENAME']
-            else:
-                filename = 'result.fits'
-            with warnings.catch_warnings():
-                warnings.simplefilter('ignore')
-                self.frame.writeto(filename, clobber=True)
+            return self.frame
 
-        return {'filename': filename, 'itype': self.itype}
-    
     @property
     def label(self):
         return self.filename
 
-    def __setstate__(self, state):
-        self.filename = state['filename']
-        self.itype = state['itype']
-        
     def __repr__(self):
         if self.frame is None:
             return "DataFrame(filename=%r)" % self.filename
@@ -66,3 +49,4 @@ class DataFrame(object):
             return "DataFrame(frame=%r)" % self.frame
         else:
             return "DataFrame(filename=%r, frame=%r)" % (self.filename, self.frame)
+
